import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 14
project_path = file_path[0:end]
sys.path.append(project_path)

from functools import lru_cache
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.utils.date_handle_util as date_handle_util
import mns_common.api.akshare.stock_zt_pool_api as stock_zt_pool_api
import mns_common.component.trade_date.trade_date_common_service_api as trade_date_common_service_api

mongodb_util = MongodbUtil('27017')


@lru_cache(maxsize=None)
def get_last_trade_day_zt(str_day):
    last_trade_day = trade_date_common_service_api.get_last_trade_day(str_day)
    query = {'str_day': last_trade_day}
    db_stock_zt_pool = mongodb_util.find_query_data('stock_zt_pool', query)
    if db_stock_zt_pool is None or db_stock_zt_pool.shape[0] == 0:
        db_stock_zt_pool = stock_zt_pool_api.stock_em_zt_pool_df(date_handle_util.no_slash_date(last_trade_day))
    return db_stock_zt_pool


if __name__ == '__main__':
    get_last_trade_day_zt('20231215')



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 7
project_path = file_path[0:end]
sys.path.append(project_path)
import akshare as ak
from loguru import logger
import os
import pandas as pd
import requests
from bs4 import BeautifulSoup
from py_mini_racer import py_mini_racer
from tqdm import tqdm
import json


def _get_js_path_ths(name: str = None, module_file: str = None) -> str:
    """
    获取 JS 文件的路径(从模块所在目录查找)
    :param name: 文件名
    :type name: str
    :param module_file: 模块路径
    :type module_file: str
    :return: 路径
    :rtype: str
    """
    module_folder = os.path.abspath(os.path.dirname(os.path.dirname(module_file)))
    module_json_path = os.path.join(module_folder, "api", name)
    return module_json_path


def _get_file_content_ths(file_name: str = "ase.min.js") -> str:
    """
    获取 JS 文件的内容
    :param file_name:  JS 文件名
    :type file_name: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_name = file_name
    setting_file_path = _get_js_path_ths(setting_file_name, __file__)
    with open(setting_file_path) as f:
        file_data = f.read()
    return file_data


"""
    同花顺-板块-概念板块-概念 获取所有概念top50 概念列表
    http://q.10jqka.com.cn/gn/detail/code/301558/
    :return: 所有概念板块的名称和链接
    :rtype: pandas.DataFrame
"""


def ths_concept_index_top_50() -> pd.DataFrame:
    try:
        url = "http://data.10jqka.com.cn/funds/gnzjl/field/tradezdf/order/desc/page/1/ajax/1/free/1/"
        js_code = py_mini_racer.MiniRacer()
        js_content = _get_file_content_ths("ths.js")
        js_code.eval(js_content)
        v_code = js_code.call('v')
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36',
            'Cookie': f'v={v_code}'
        }
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.content.decode("gbk"), 'lxml')

        big_df = pd.DataFrame()

        soup.find('table', attrs={'class': 'm-table J-ajax-table'}).find('tbody')
        url_list = []
        for item in soup.find('table', attrs={'class': 'm-table J-ajax-table'}).find('tbody').find_all('tr'):
            inner_url = item.find_all("td")[1].find('a')['href']
            url_list.append(inner_url)
        temp_df = pd.read_html(r.text)[0]
        temp_df['网址'] = url_list
        big_df = big_df.append(temp_df, ignore_index=True)
        big_df['代码'] = big_df['网址'].str.split("/", expand=True).iloc[:, 6]
        big_df.rename(columns={
            "排名": "index",
            "代码": "concept_code",
            "行业": "concept_name",
            "行业指数": "now_price",
            "涨跌幅": "chg",
            "流入资金(亿)": "inflows",
            "流出资金(亿)": "outflows",
            "净额(亿)": "net_flows",
            "公司家数": "company_num",
            "领涨股": "first_up_symbol_name",
            "涨跌幅.1": "first_up_symbol_chg",
            "当前价(元)": "first_up_symbol_now_price",
            "网址": "url"
        }, inplace=True)
        big_df.loc[big_df['now_price'] == '-', 'now_price'] = 0
        big_df.loc[big_df['chg'] == '-', 'chg'] = 0
        big_df.loc[big_df['inflows'] == '-', 'inflows'] = 0
        big_df.loc[big_df['outflows'] == '-', 'outflows'] = 0
        big_df.loc[big_df['net_flows'] == '-', 'net_flows'] = 0
        big_df.loc[big_df['company_num'] == '-', 'company_num'] = 0
        big_df.loc[big_df['first_up_symbol_chg'] == '-', 'first_up_symbol_chg'] = 0
        big_df.loc[big_df['first_up_symbol_now_price'] == '-', 'first_up_symbol_now_price'] = 0
        big_df['chg'] = big_df['chg'].apply(
            lambda x: x.replace("%", "0"))
        big_df['first_up_symbol_chg'] = big_df['first_up_symbol_chg'].apply(
            lambda x: x.replace("%", "0"))
        big_df["now_price"] = pd.to_numeric(big_df["now_price"])
        big_df["chg"] = pd.to_numeric(big_df["chg"])
        big_df["inflows"] = pd.to_numeric(big_df["inflows"])
        big_df["outflows"] = pd.to_numeric(big_df["outflows"])
        big_df["net_flows"] = pd.to_numeric(big_df["net_flows"])
        big_df["company_num"] = pd.to_numeric(big_df["company_num"])
        big_df["first_up_symbol_chg"] = pd.to_numeric(big_df["first_up_symbol_chg"])
        big_df["first_up_symbol_now_price"] = pd.to_numeric(big_df["first_up_symbol_now_price"])
        return big_df
    except Exception as e:
        logger.error("获取同花顺概念指数异常:{}", e)
        return None


""" 有延迟 通过资金板块获取
    同花顺-板块-概念板块-概念 获取所有概念列表
    http://q.10jqka.com.cn/gn/detail/code/301558/
    :return: 所有概念板块的名称和链接
    :rtype: pandas.DataFrame
"""


def ths_concept_index() -> pd.DataFrame:
    try:
        url = "http://data.10jqka.com.cn/funds/gnzjl/field/tradezdf/order/desc/page/1/ajax/1/free/1/"
        js_code = py_mini_racer.MiniRacer()
        js_content = _get_file_content_ths("ths.js")
        js_code.eval(js_content)
        v_code = js_code.call('v')
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36',
            'Cookie': f'v={v_code}'
        }
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.content.decode("gbk"), 'lxml')
        total_page = soup.find('span', attrs={'class': 'page_info'}).text.split('/')[1]
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page) + 1), leave=False):
            url = f"http://data.10jqka.com.cn/funds/gnzjl/field/tradezdf/order/desc/page/{page}/ajax/1/free/1/"

            js_code = py_mini_racer.MiniRacer()
            js_content = _get_file_content_ths("ths.js")
            js_code.eval(js_content)
            v_code = js_code.call('v')
            headers = {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36',
                'Cookie': f'v={v_code}'
            }
            r = requests.get(url, headers=headers)
            soup = BeautifulSoup(r.content.decode("gbk"), 'lxml')
            soup.find('table', attrs={'class': 'm-table J-ajax-table'}).find('tbody')
            url_list = []
            for item in soup.find('table', attrs={'class': 'm-table J-ajax-table'}).find('tbody').find_all('tr'):
                inner_url = item.find_all("td")[1].find('a')['href']
                url_list.append(inner_url)
            temp_df = pd.read_html(r.text)[0]
            temp_df['网址'] = url_list
            big_df = big_df.append(temp_df, ignore_index=True)
        big_df['代码'] = big_df['网址'].str.split("/", expand=True).iloc[:, 6]
        big_df.rename(columns={
            "序号": "index",
            "代码": "concept_code",
            "行业": "concept_name",
            "行业指数": "now_price",
            "涨跌幅": "chg",
            "流入资金(亿)": "inflows",
            "流出资金(亿)": "outflows",
            "净额(亿)": "net_flows",
            "公司家数": "company_num",
            "领涨股": "first_up_symbol_name",
            "涨跌幅.1": "first_up_symbol_chg",
            "当前价(元)": "first_up_symbol_now_price",
            "网址": "url"
        }, inplace=True)
        big_df.loc[big_df['now_price'] == '-', 'now_price'] = 0
        big_df.loc[big_df['chg'] == '-', 'chg'] = 0
        big_df.loc[big_df['inflows'] == '-', 'inflows'] = 0
        big_df.loc[big_df['outflows'] == '-', 'outflows'] = 0
        big_df.loc[big_df['net_flows'] == '-', 'net_flows'] = 0
        big_df.loc[big_df['company_num'] == '-', 'company_num'] = 0
        big_df.loc[big_df['first_up_symbol_chg'] == '-', 'first_up_symbol_chg'] = 0
        big_df.loc[big_df['first_up_symbol_now_price'] == '-', 'first_up_symbol_now_price'] = 0
        big_df['chg'] = big_df['chg'].apply(
            lambda x: x.replace("%", "0"))
        big_df['first_up_symbol_chg'] = big_df['first_up_symbol_chg'].apply(
            lambda x: x.replace("%", "0"))
        big_df["now_price"] = pd.to_numeric(big_df["now_price"])
        big_df["chg"] = pd.to_numeric(big_df["chg"])
        big_df["inflows"] = pd.to_numeric(big_df["inflows"])
        big_df["outflows"] = pd.to_numeric(big_df["outflows"])
        big_df["net_flows"] = pd.to_numeric(big_df["net_flows"])
        big_df["company_num"] = pd.to_numeric(big_df["company_num"])
        big_df["first_up_symbol_chg"] = pd.to_numeric(big_df["first_up_symbol_chg"])
        big_df["first_up_symbol_now_price"] = pd.to_numeric(big_df["first_up_symbol_now_price"])
        return big_df
    except Exception as e:
        logger.error("获取同花顺概念指数异常:{}", e)
        return None


# 同步同花顺所有概念模块 通过概念板块
def stock_board_concept_name_ths():
    try:
        stock_board_industry_name_ths = ak.stock_board_concept_name_ths()
        stock_board_industry_name_ths = stock_board_industry_name_ths.rename(columns={"日期": "create_date",
                                                                                      "概念名称": "concept_name",
                                                                                      "成分股数量": "company_num",
                                                                                      "网址": "url",
                                                                                      "代码": "concept_code"})
        stock_board_industry_name_ths['_id'] = stock_board_industry_name_ths['concept_code']
        stock_board_industry_name_ths.create_date = stock_board_industry_name_ths.create_date.astype(str)
        return stock_board_industry_name_ths
    except Exception as e:
        logger.error("获取同花顺概念异常:{}", e)
        return None


"""
 概念板块的成份股 根据概念代码获取
"""


def ths_concept_symbol_list(symbol: str = "305794") -> pd.DataFrame:
    try:
        """
        http://q.10jqka.com.cn/gn/detail/code/305794/
        :param symbol: 行业板块或者概念板块的代码
        :type symbol: str
        :return: 概念板块的成份股
        :rtype: pandas.DataFrame
        """
        js_code = py_mini_racer.MiniRacer()
        js_content = _get_file_content_ths("ths.js")
        js_code.eval(js_content)
        v_code = js_code.call('v')
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36',
            'Cookie': f'v={v_code}'
        }
        url = f'http://q.10jqka.com.cn/gn/detail/field/264648/order/desc/page/1/ajax/1/code/{symbol}'
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.text, "lxml")
        try:
            page_num = int(soup.find_all('a', attrs={'class': 'changePage'})[-1]['page'])
        except IndexError as e:
            page_num = 1
        big_df = pd.DataFrame()
        for page in tqdm(range(1, page_num + 1), leave=False):
            v_code = js_code.call('v')
            headers = {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36',
                'Cookie': f'v={v_code}'
            }
            url = f'http://q.10jqka.com.cn/gn/detail/field/264648/order/desc/page/{page}/ajax/1/code/{symbol}'
            r = requests.get(url, headers=headers)
            temp_df = pd.read_html(r.text)[0]
            big_df = big_df.append(temp_df, ignore_index=True)
        big_df.rename({
            "序号": "index",
            "代码": "symbol",
            "名称": "name",
            "现价": "now_price",
            "涨跌": "change",
            "量比": "quantity_ratio",
            "成交额": "amount",
            "流通股": "flow_shares",
            "流通市值": "flow_mv",
            "市盈率": "pe",
            "涨跌幅(%)": "chg",
            "涨速(%)": "speed_chg",
            "换手(%)": "exchange",
            "振幅(%)": "pct_chg",
        }, inplace=True, axis=1)
        del big_df['加自选']
        big_df = big_df[
            big_df["index"] != '暂无成份股数据']
        big_df['symbol'] = big_df['symbol'].astype(str).str.zfill(6)
        big_df.loc[big_df['now_price'] == '-', 'now_price'] = 0
        big_df.loc[big_df['change'] == '-', 'change'] = 0
        big_df.loc[big_df['quantity_ratio'] == '-', 'quantity_ratio'] = 0
        big_df.loc[big_df['amount'] == '-', 'amount'] = 0
        big_df.loc[big_df['flow_shares'] == '-', 'flow_shares'] = 0
        big_df.loc[big_df['flow_mv'] == '-', 'flow_mv'] = 0
        big_df.loc[big_df['pe'] == '-', 'pe'] = 0
        big_df.loc[big_df['chg'] == '-', 'chg'] = 0
        big_df.loc[big_df['speed_chg'] == '-', 'speed_chg'] = 0
        big_df.loc[big_df['exchange'] == '-', 'exchange'] = 0
        big_df.loc[big_df['pct_chg'] == '-', 'pct_chg'] = 0

        big_df.loc[big_df['now_price'] == '--', 'now_price'] = 0
        big_df.loc[big_df['change'] == '--', 'change'] = 0
        big_df.loc[big_df['quantity_ratio'] == '--', 'quantity_ratio'] = 0
        big_df.loc[big_df['amount'] == '--', 'amount'] = 0
        big_df.loc[big_df['flow_shares'] == '--', 'flow_shares'] = 0
        big_df.loc[big_df['flow_mv'] == '--', 'flow_mv'] = 0
        big_df.loc[big_df['pe'] == '--', 'pe'] = 0
        big_df.loc[big_df['chg'] == '--', 'chg'] = 0
        big_df.loc[big_df['speed_chg'] == '--', 'speed_chg'] = 0
        big_df.loc[big_df['exchange'] == '--', 'exchange'] = 0
        big_df.loc[big_df['pct_chg'] == '--', 'pct_chg'] = 0
        big_df.amount = big_df.amount.astype(str)
        big_df.flow_shares = big_df.flow_shares.astype(str)
        big_df.flow_mv = big_df.flow_mv.astype(str)
        big_df['amount'] = big_df['amount'].apply(
            lambda x: x.replace("亿", ""))
        big_df['flow_shares'] = big_df['flow_shares'].apply(
            lambda x: x.replace("亿", ""))
        big_df['flow_mv'] = big_df['flow_mv'].apply(
            lambda x: x.replace("亿", ""))

        big_df["now_price"] = pd.to_numeric(big_df["now_price"])

        big_df["change"] = pd.to_numeric(big_df["change"])

        big_df["quantity_ratio"] = pd.to_numeric(big_df["quantity_ratio"])
        big_df["amount"] = pd.to_numeric(big_df["amount"])

        big_df["flow_shares"] = pd.to_numeric(big_df["flow_shares"])

        big_df["flow_mv"] = pd.to_numeric(big_df["flow_mv"])

        big_df["pe"] = pd.to_numeric(big_df["pe"])

        big_df["chg"] = pd.to_numeric(big_df["chg"])

        big_df["speed_chg"] = pd.to_numeric(big_df["speed_chg"])

        big_df["exchange"] = pd.to_numeric(big_df["exchange"])

        big_df["pct_chg"] = pd.to_numeric(big_df["pct_chg"])

        return big_df
    except Exception as e:
        logger.error("获取同花顺概念列表信息异常:{}", e)
        return None


# 根据名称获取概念组成详细信息
def ths_concept_name_detail(concept_name):
    try:
        stock_board_concept_cons_ths_df = ak.stock_board_concept_cons_ths(
            symbol=concept_name)
        stock_board_concept_cons_ths_df.rename(columns={"序号": "index",
                                                        "代码": "symbol",
                                                        "名称": "name",
                                                        "现价": "now_price",
                                                        "涨跌幅": "chg",
                                                        "涨跌": "change",
                                                        "涨速": "r_increase",
                                                        "换手": "exchange",
                                                        "量比": "q_ratio",
                                                        "振幅": "pct_chg",
                                                        "成交额": "amount",
                                                        "流通股": "tradable_shares",
                                                        "流通市值": "market_value",
                                                        "市盈率": "pe"
                                                        }, inplace=True)
        stock_board_concept_cons_ths_df = stock_board_concept_cons_ths_df[
            stock_board_concept_cons_ths_df["index"] != '暂无成份股数据']
        stock_board_concept_cons_ths_df.chg = stock_board_concept_cons_ths_df.chg.astype(str)
        stock_board_concept_cons_ths_df['chg'] = stock_board_concept_cons_ths_df['chg'].apply(
            lambda x: x.replace("--", "0"))
        stock_board_concept_cons_ths_df.chg = stock_board_concept_cons_ths_df.chg.astype(float)
        return stock_board_concept_cons_ths_df
    except Exception as e:
        logger.error("获取同花顺概念详细信息异常:{}", e)
        return None


def ths_stock_concept(code):
    try:
        stock_board_cons_ths_df = ak.stock_board_cons_ths(symbol=code)
        if stock_board_cons_ths_df is None:
            return
        stock_board_cons_ths_df.rename(columns={"序号": "index",
                                                "代码": "symbol",
                                                "名称": "name",
                                                "现价": "now_price",
                                                "涨跌幅": "chg",
                                                "涨跌": "change",
                                                "涨速": "r_increase",
                                                "换手": "exchange",
                                                "量比": "q_ratio",
                                                "振幅": "pct_chg",
                                                "成交额": "amount",
                                                "流通股": "tradable_shares",
                                                "流通市值": "flow_mv",
                                                "市盈率": "pe"
                                                }, inplace=True)
        stock_board_cons_ths_df = stock_board_cons_ths_df[
            stock_board_cons_ths_df["index"] != '暂无成份股数据']

        if stock_board_cons_ths_df is None or stock_board_cons_ths_df.shape[0] == 0:
            return
        length = len(list(stock_board_cons_ths_df))
        stock_board_cons_ths_df.insert(length, 'concept_code', code)

        stock_board_cons_ths_df['amount'] = stock_board_cons_ths_df['amount'].apply(
            lambda x: pd.to_numeric(x.replace('亿', ''), errors="coerce"))

        stock_board_cons_ths_df['tradable_shares'] = stock_board_cons_ths_df['tradable_shares'].apply(
            lambda x: pd.to_numeric(x.replace('亿', ''), errors="coerce"))

        stock_board_cons_ths_df['flow_mv'] = stock_board_cons_ths_df['flow_mv'].apply(
            lambda x: pd.to_numeric(x.replace('亿', ''), errors="coerce"))

        return stock_board_cons_ths_df
        logger.info("同步概念代码数据:{}", code)
    except BaseException as e:
        logger.error("获取详情异常:{},异常信息:{}", code, e)


def get_company_info_detail(symbol: str = "305794") -> pd.DataFrame:
    try:
        url = f'https://basic.10jqka.com.cn/basicapi/company_info/merge_info/v1/base_info/?code={symbol}&market=33&type=stock'
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:105.0) Gecko/20100101 Firefox/105.0',
            'Host': 'basic.10jqka.com.cn',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8',
            'Accept-Language': 'zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2',
            'Accept-Encoding': 'gzip, deflate, br',
            'Connection': 'keep-alive',
            'Cookie': 'v=A0CN8EBXN21LtMtpV6ldAxf6Ec8XySSbxq14l7rRDNvuNe77Ytn0Ixa9SDQJ',
            'Upgrade-Insecure-Requests': '1',

        }
        r = requests.get(url, headers=headers)
        data_json = r.json()
        status_code = data_json['status_code']
        status_msg = data_json['status_msg']
        if status_code != 0 or status_msg != 'success':
            logger.error("获取symbol公司详细信息异常:{}", symbol)
        if len(data_json['data']['industry']) == 0:
            return None
        data_df = pd.DataFrame(data_json['data']['industry'], index=[0])
        data_df = data_df[[
            'hy',
            'hycode',
            'hy2',
            'hy2code',
            'hy3',
            'hy3code',
        ]]

        business_nature = data_json['data']['business_nature']
        name = data_json['data']['code_name']
        base_business = data_json['data']['base_business']
        address = data_json['data']['address']
        data_df['symbol'] = symbol
        data_df['name'] = name

        data_df['business_nature'] = business_nature

        if len(data_json['data']['management']['holder_controller']) > 0:
            holder_controller = pd.DataFrame(data_json['data']['management']['holder_controller'])
            holder_controller_name = str(list(holder_controller['name'])).strip('[').strip(']').replace("'", "")
            holder_controller_rate = holder_controller['rate']
            data_df['holder_controller_name'] = holder_controller_name
            data_df['holder_controller_rate'] = sum(holder_controller_rate)
        else:
            data_df['holder_controller_name'] = '暂无'
            data_df['holder_controller_rate'] = 0
        if len(data_json['data']['management']['final_controller']) > 0:
            final_controller = pd.DataFrame(data_json['data']['management']['final_controller'])
            final_controller_name = str(list(final_controller['name'])).strip('[').strip(']').replace("'", "")
            final_controller_rate = sum(final_controller['rate'])
            data_df['final_controller_name'] = final_controller_name
            data_df['final_controller_rate'] = final_controller_rate
        else:
            data_df['final_controller_name'] = '暂无'
            data_df['final_controller_rate'] = 0
        if len(data_json['data']['management']['actual_controller']) > 0:
            actual_controller = pd.DataFrame(data_json['data']['management']['actual_controller'])
            actual_controller_name = str(list(actual_controller['name'])).strip('[').strip(']').replace("'", "")
            actual_controller_rate = sum(actual_controller['rate'])
            data_df['actual_controller_name'] = actual_controller_name
            data_df['actual_controller_rate'] = actual_controller_rate
        else:
            data_df['actual_controller_name'] = '暂无'
            data_df['actual_controller_rate'] = 0

        data_df['base_business'] = base_business
        data_df['address'] = address
        market_id = data_json['data']['market_id']
        data_df['market_id'] = market_id

        return data_df
    except BaseException as e:
        logger.error("获取symbol公司详细信息异常:{},{}", symbol, e)


# 获取股票基本信息
# https://basic.10jqka.com.cn/mobile/301016/companyprofilen.html?showtab=1&broker=anelicaiapp
def get_company_info(symbol: str = "305794") -> pd.DataFrame:
    try:
        url = f"http://basic.10jqka.com.cn/mobile/{symbol}/companyprofilen.html?broker=pingan"
        headers = {
            "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 15_4_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 iOS AYLCAPP/9.1.2.0/h4526a24eb9445522492fd64caae11b1f scheme/anelicaiapp deviceinfo/I|9.1.2.0|NA|h4526a24eb9445522492fd64caae11b1f pastheme/0",
            "Cookie": "ps_login_app_name=AYLCAPP;ps_login_token_id=N_C993F777ACC500B354C762A2627A8862348FC8163799A08EBEB2301C28A2135D220475787D0E81425C1134E15D8CC8761D639FEDBD46C00FE8EA6482C1E42D9801B19918FB3F5C34;ps_login_union_id=edc29089a2b64e3882062297030a0386;PAS.CURRENTUNIONID=edc29089a2b64e3882062297030a0386"
        }
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.content.decode("utf-8"), 'lxml')
        soup.find('table', attrs={'class': 'leveldatail-tab'}).find_all('tr')
        temp_df = pd.read_html(r.content)[0]
        temp_df = temp_df.T
        temp_df = temp_df.iloc[1:2]
        temp_df.rename(columns={
            0: "name",
            1: "former_name",
            2: "registered_address",
            3: "chairman",
            4: "board_secretary",
            5: "main_business",
            6: "company_type",
            7: "controlling_shareholder",
            8: "actual_controller",
            9: "ultimate_controller",
            10: "list_date",
            11: "issue_price",
            12: "number_workers",
            13: "tel",
            14: "url",
            15: "email"
        }, inplace=True)

        return temp_df
    except BaseException:
        logger.error("获取symbol控制人基本信息异常:{}", symbol)


# 获取概念名称信息
def get_concept_name(symbol: str = "881121") -> pd.DataFrame:
    try:
        url = f"http://q.10jqka.com.cn/thshy/detail/code/{symbol}/"
        headers = {
            "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 15_4_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 iOS AYLCAPP/9.1.2.0/h4526a24eb9445522492fd64caae11b1f scheme/anelicaiapp deviceinfo/I|9.1.2.0|NA|h4526a24eb9445522492fd64caae11b1f pastheme/0",
            "Cookie": "ps_login_app_name=AYLCAPP;ps_login_token_id=N_C993F777ACC500B354C762A2627A8862348FC8163799A08EBEB2301C28A2135D220475787D0E81425C1134E15D8CC8761D639FEDBD46C00FE8EA6482C1E42D9801B19918FB3F5C34;ps_login_union_id=edc29089a2b64e3882062297030a0386;PAS.CURRENTUNIONID=edc29089a2b64e3882062297030a0386"
        }
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.content.decode("gbk"), 'lxml')
        temp_df = soup.find(attrs={'class': 'board-hq'}).find_all('h3')
        temp_df = str(temp_df)
        startIndex = temp_df.index('[<h3>')
        if startIndex >= 0:
            startIndex += len('[<h3>')
        endIndex = temp_df.index('<span>')
        return temp_df[startIndex:endIndex]

    except BaseException:
        logger.error("获取symbol基本信息异常:{}", symbol)


# 获取概念资金流入信息
def get_stock_fund_flow():
    stock_fund_flow_concept_df = ak.stock_fund_flow_concept(symbol="即时")
    stock_fund_flow_concept_df = stock_fund_flow_concept_df.rename(columns={
        "序号": "index",
        "行业": "concept_name",
        "行业指数": "concept_index",
        "行业-涨跌幅": "concept_chg",
        "流入资金": "inflows",
        "流出资金": "outflow",
        "净额": "netflow",
        "公司家数": "company_num",
        "领涨股": "leading_stock",
        "领涨股-涨跌幅": "leading_chg",
        "当前价": "now_price",
    })
    return stock_fund_flow_concept_df


# 获取同花顺新概念信息--APP 端接口 容易被封
def get_new_concept_list_app(symbol: str = "886019") -> pd.DataFrame:
    try:
        url = f'https://d.10jqka.com.cn/v2/blockrank/{symbol}/199112/d1000.js'
        headers = {

            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8',
            'Accept-Language': 'zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2',
            'Accept-Encoding': 'gzip, deflate, br',
            'Connection': 'keep-alive',
            'Cookie': 'v=A2A7Vv5CF9c_javTJ4_ZJx3_N283aUQt5k2YN9pxLHsO1Q5bgnkUwzZdaMEp; Hm_lvt_78c58f01938e4d85eaf619eae71b4ed1=1668578108',
            'Host': 'd.10jqka.com.cn',
            'If-Modified-Since': "Wed, 16 Nov 2022 09:31:32 GMT",
            'Sec-Fetch-Dest': 'document',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-Site': 'none',
            'Sec-Fetch-User': '?1',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:106.0) Gecko/20100101 Firefox/106.0",

        }
        r = requests.get(url, headers=headers)
        result = r.content.decode("unicode-escape")

        startIndex = result.index('{"block"')
        endIndex = result.index('}]}')

        result = result[startIndex:endIndex + 3]

        data_json = json.loads(result)
        block = data_json['block']
        items = data_json['items']

        items_df = pd.DataFrame(items)
        items_df = items_df.rename(columns={
            "5": "symbol",
            "199112": "chg",
            "264648": "change",
            "2034120": "pe",
            "3475914": "flow_mv",
            "3541450": "total_mv",
            "1968584": "exchange",
            "10": "now_price",
            "7": "open",
            "8": "high",
            "9": "low",
            "13": "volume",
            "19": "amount",
            "55": "name",
            "6": "last_day_price",

        })
        items_df['concept_code'] = symbol
        items_df['concept_name'] = block['name']
        items_df['index'] = 0
        items_df['pct_chg'] = 0
        items_df['q_ratio'] = 0
        items_df = items_df[
            [
                'index',
                'symbol',
                'name',
                'now_price',
                'chg',
                'change',
                'exchange',
                'q_ratio',
                'pct_chg',
                'amount',
                'flow_mv',
                'total_mv',
                'pe',
                'concept_code',
                'concept_name',
            ]
        ]
        return items_df
    except BaseException:
        logger.error("获取新概念信息异常:{}", symbol)
        return None


# 获取同花顺APP 端搜索概念结果
def get_new_concept_code_app(symbol: str = "886019"):
    headers = {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8',
        'Accept-Language': 'zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2',
        'Accept-Encoding': 'gzip, deflate, br',
        'Connection': 'keep-alive',
        'Cookie': 'v=A2A7Vv5CF9c_javTJ4_ZJx3_N283aUQt5k2YN9pxLHsO1Q5bgnkUwzZdaMEp; Hm_lvt_78c58f01938e4d85eaf619eae71b4ed1=1668578108',
        'Host': 'd.10jqka.com.cn',
        'If-Modified-Since': "Wed, 16 Nov 2022 09:31:32 GMT",
        'Sec-Fetch-Dest': 'document',
        'Sec-Fetch-Mode': 'navigate',
        'Sec-Fetch-Site': 'none',
        'Sec-Fetch-User': '?1',
        'Upgrade-Insecure-Requests': '1',
        'User-Agent': "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:106.0) Gecko/20100101 Firefox/106.0",

    }
    # userId 随便写 https://dict.hexin.cn:9531/stocks?pattern=886019&isauto=1&associate=1&pl=i&isrealcode=1&json=1&br=sc&style=1&userid=656258250&markettype=2	200	GET	dict.hexin.cn:9531	/stocks?pattern=886019&isauto=1&associate=1&pl=i&isrealcode=1&json=1&br=sc&style=1&userid=656258250&markettype=2
    url = f'https://dict.hexin.cn:9531/stocks?pattern={symbol}&isauto=1&associate=1&pl=i&isrealcode=1&json=1&br=sc&style=1&userid=603985000&markettype=2'

    try:
        r = requests.get(url, headers)
        data_json = r.json()
        type = data_json['type']
        if type == '0':

            body_list = data_json['data']['body']
            concept_code = body_list[0][0]
            concept_name = body_list[0][1]
            concept = {
                'concept_code': concept_code,
                'concept_name': concept_name
            }

            return pd.DataFrame([concept], index=[0])

            return concept
        else:
            return None

    except BaseException as e:
        logger.error("获取新概念信息异常:{}", e)
        return None


# 获取单个股票新增概念  https://basic.10jqka.com.cn/basicph/briefinfo.html#/concept?broker=anelicaiapp&showtab=1&code=301016&code_name=%E9%9B%B7%E5%B0%94%E4%BC%9F&market_id=33
def get_symbol_add_new_concept(symbol: str = "305794") -> pd.DataFrame:
    try:
        url = f"http://basic.10jqka.com.cn/api/stockph/conceptdetail/{symbol}"
        headers = {
            "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 15_4_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 iOS AYLCAPP/9.1.2.0/h4526a24eb9445522492fd64caae11b1f scheme/anelicaiapp deviceinfo/I|9.1.2.0|NA|h4526a24eb9445522492fd64caae11b1f pastheme/0",
            "Cookie": "ps_login_app_name=AYLCAPP;ps_login_token_id=N_C993F777ACC500B354C762A2627A8862348FC8163799A08EBEB2301C28A2135D220475787D0E81425C1134E15D8CC8761D639FEDBD46C00FE8EA6482C1E42D9801B19918FB3F5C34;ps_login_union_id=edc29089a2b64e3882062297030a0386;PAS.CURRENTUNIONID=edc29089a2b64e3882062297030a0386"
        }
        r = requests.get(url, headers=headers)
        data_json = r.json()
        data_concept = data_json['data']
        return data_concept
    except BaseException:
        logger.error("获取symbol概念信息异常:{}", symbol)


# query: 个股新增概念
# urp_sort_way: desc
# urp_sort_index: 最新涨跌幅
# page: 2
# perpage: 50
# addheaderindexes:
# condition: [{"dateText":"","indexName":"所属概念","indexProperties":[],"ci":false,"source":"text2sql","type":"index","indexPropertiesMap":{},"reportType":"null","ciChunk":"个股新增概念","score":0.0,"createBy":"cache","chunkedResult":"个股新增概念","uiText":"所属概念","valueType":"_所属概念","domain":"abs_股票领域","sonSize":0,"logid":"78951d1a2de43d989afb6f5fff55538f","order":0}]
# codelist:
# indexnamelimit:
# logid: 78951d1a2de43d989afb6f5fff55538f
# ret: json_all
# sessionid: 78951d1a2de43d989afb6f5fff55538f
# source: Ths_iwencai_Xuangu
# date_range[0]: 20231031
# iwc_token: 0ac9665916987651304637922
# urp_use_sort: 1
# user_id: Ths_iwencai_Xuangu_xqei21psb40ujl9i60sjfv752bkkumeq
# uuids[0]: 24087
# query_type: stock
# comp_id: 6836372
# business_cat: soniu
# uuid: 24087
def ths_concept_wen_cai(str_day, page_number):
    try:
        # 请求头信息
        headers = {
            "Host": "www.iwencai.com",
            "Connection": "keep-alive",
            "Content-Length": "1326",
            "sec-ch-ua": "\"Chromium\";v=\"118\", \"Google Chrome\";v=\"118\", \"Not=A?Brand\";v=\"99\"",
            "Pragma": "no-cache",
            "hexin-v": "AzR_iz347VCrfXnqX7tBEAuEBfmjDUzPmj3sFs6CwAxDFdon9h0oh-pBvO8d",
            "sec-ch-ua-mobile": "?0",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36",
            "Content-Type": "application/x-www-form-urlencoded",
            "Accept": "application/json, text/plain, */*",
            "Cache-control": "no-cache",
            "sec-ch-ua-platform": "\"Windows\"",
            "Origin": "https://www.iwencai.com",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": "https://www.iwencai.com/unifiedwap/result?w=%E4%B8%AA%E8%82%A1%E6%96%B0%E5%A2%9E%E6%A6%82%E5%BF%B5",
            "Accept-Encoding": "gzip, deflate, br",
            "Accept-Language": "zh-CN,zh;q=0.9",
            "Cookie": "other_uid=Ths_iwencai_Xuangu_xqei21psb40ujl9i60sjfv752bkkumeq; ta_random_userid=zw66u7re6z; cid=78ed13c5d7b1b7d572f3b515b3dd9f7a1694788715; v=AzR_iz347VCrfXnqX7tBEAuEBfmjDUzPmj3sFs6CwAxDFdon9h0oh-pBvO8d"
        }

        # 请求体数据
        data = {
            "query": "个股新增概念",
            "urp_sort_way": "desc",
            "urp_sort_index": "最新涨跌幅",
            "page": page_number,
            "perpage": "50",
            "addheaderindexes": "",
            "condition": "[{\"dateText\":\"\",\"indexName\":\"所属概念\",\"indexProperties\":[],\"ci\":false,\"source\":\"text2sql\",\"type\":\"index\",\"indexPropertiesMap\":{},\"reportType\":\"null\",\"ciChunk\":\"个股新增概念\",\"score\":0.0,\"createBy\":\"cache\",\"chunkedResult\":\"个股新增概念\",\"uiText\":\"所属概念\",\"valueType\":\"_所属概念\",\"domain\":\"abs_股票领域\",\"sonSize\":0,\"logid\":\"78951d1a2de43d989afb6f5fff55538f\",\"order\":0}]",
            "codelist": "",
            "indexnamelimit": "",
            "logid": "78951d1a2de43d989afb6f5fff55538f",
            "ret": "json_all",
            "sessionid": "78951d1a2de43d989afb6f5fff55538f",
            "source": "Ths_iwencai_Xuangu",
            "date_range[0]": str_day,
            "iwc_token": "0ac9665916987651304637922",
            "urp_use_sort": "1",
            "user_id": "Ths_iwencai_Xuangu_xqei21psb40ujl9i60sjfv752bkkumeq",
            "uuids[0]": "24087",
            "query_type": "stock",
            "comp_id": "6836372",
            "business_cat": "soniu",
            "uuid": "24087"
        }

        # 发送POST请求
        url = "https://www.iwencai.com/gateway/urp/v7/landing/getDataList"
        response = requests.post(url, headers=headers, data=data)
        # 输出响应内容
        data_json = response.json()
        data_stock = data_json['answer']['components'][0]['data']['datas']
        data_stock_list = pd.DataFrame(data_stock)
        return data_stock_list
    except BaseException:
        logger.error("同步问财信息异常:{}", str_day, page_number)


if __name__ == '__main__':
    df = ths_stock_concept('885598')
    print(df)
    # while True:
    #     concept_test()
    # company_info = get_concept_name('881121')
    # print(company_info)
    # new_concept_symbol_list = ths_stock_concept('881121')
    # print(new_concept_symbol_list)
    #
    # stock_fund_flow_concept_df = get_stock_fund_flow()
    # print(stock_fund_flow_concept_df)
    # df = get_company_info_detail("300085")
    # print(df)
    # while True:
    #     new_concept_code = get_new_concept_code_app('886026')
    #     new_concept_symbol_list = get_new_concept_list_app('886026')
    #     logger.info(new_concept_symbol_list)



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 7
project_path = file_path[0:end]
sys.path.append(project_path)

BIG_DEAL_NAME = "ths_big_deal_fund"
BIG_DEAL_CHOOSE_NAME = "big_deal_fund_choose"



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 7
project_path = file_path[0:end]
sys.path.append(project_path)
import akshare as ak
import mns_common.component.common_service_fun_api as common_service_fun_api
import mns_common.utils.date_handle_util as date_handle_util
from loguru import logger
import mns_common.api.em.east_money_stock_api as east_money_stock_api
from mns_common.db.MongodbUtil import MongodbUtil

mongodb_util = MongodbUtil('27017')


def get_stock_gdfx_free_top_10_em_api(str_day, symbol):
    try:
        stock_gdfx_free_top_10_em_df = ak.stock_gdfx_free_top_10_em(symbol=symbol, date=str_day)
        stock_gdfx_free_top_10_em_df.rename(columns={
            "名次": "index",
            "股东名称": "shareholder_name",
            "股东性质": "shareholder_nature",
            "股份性质": "shares_nature",
            "股份类型": "shares_type",
            "持股数": "shares_number",
            "占总流通股本持股比例": "circulation_ratio",
            "增减": "change",
            "变动比率": "change_ratio"
        }, inplace=True)
    except BaseException as e:
        return None
    stock_gdfx_free_top_10_em_df = stock_gdfx_free_top_10_em_df.fillna(0)
    stock_gdfx_free_top_10_em_df.index = stock_gdfx_free_top_10_em_df.index.astype(str)
    return stock_gdfx_free_top_10_em_df


def get_stock_gdfx_free_top_10_em(str_day, symbol):
    symbol_init = symbol
    classification = common_service_fun_api.classify_symbol_one(symbol)
    if classification in ["S", "C"]:
        symbol = 'sz' + symbol
    elif classification in ["K", "H"]:
        symbol = 'sh' + symbol
    else:
        return

    str_day_no_slash = date_handle_util.no_slash_date(str_day)
    date_day = date_handle_util.str_to_date(str_day_no_slash, '%Y%m%d')
    month = date_day.month
    year = date_day.year
    one = '0331'
    two = '0630'
    three = '0930'
    four = '1231'

    if 0 < month <= 4:
        period_04 = str(year - 1) + four
        stock_gdfx_free_top_10_04 = get_stock_gdfx_free_top_10_em_api(period_04, symbol)
        sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_04, period_04, symbol_init, str_day)

        if stock_gdfx_free_top_10_04 is None or stock_gdfx_free_top_10_04.shape[0] == 0:
            period_03 = str(year - 1) + three
            stock_gdfx_free_top_10_03 = get_stock_gdfx_free_top_10_em_api(period_03, symbol)
            sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_03, period_03, symbol_init, str_day)

        period_01 = str(year) + one
        stock_gdfx_free_top_10_01 = get_stock_gdfx_free_top_10_em_api(period_01, symbol)
        sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_01, period_01, symbol_init, str_day)
    elif 4 < month <= 6:
        period_01 = str(year) + one
        stock_gdfx_free_top_10_01 = get_stock_gdfx_free_top_10_em_api(period_01, symbol)
        sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_01, period_01, symbol_init, str_day)
        period_02 = str(year) + two
        stock_gdfx_free_top_10_02 = get_stock_gdfx_free_top_10_em_api(period_02, symbol)
        sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_02, period_02, symbol_init, str_day)
    elif 6 < month <= 10:
        period_02 = str(year) + two
        stock_gdfx_free_top_10_02 = get_stock_gdfx_free_top_10_em_api(period_02, symbol)
        sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_02, period_02, symbol_init, str_day)
        period_03 = str(year) + three
        stock_gdfx_free_top_10_03 = get_stock_gdfx_free_top_10_em_api(period_03, symbol)
        sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_03, period_03, symbol_init, str_day)
    elif 10 < month <= 12:
        period_03 = str(year) + three
        stock_gdfx_free_top_10_03 = get_stock_gdfx_free_top_10_em_api(period_03, symbol)
        sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_03, period_03, symbol_init, str_day)
        period_04 = str(year) + four
        stock_gdfx_free_top_10_04 = get_stock_gdfx_free_top_10_em_api(period_04, symbol)
        sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_04, period_04, symbol_init, str_day)


def sync_stock_gdfx_free_top_10(stock_gdfx_free_top_10_em_df, period, symbol, str_day):
    if stock_gdfx_free_top_10_em_df is not None and stock_gdfx_free_top_10_em_df.shape[0] > 0:
        stock_gdfx_free_top_10_em_df['str_day'] = str_day
        stock_gdfx_free_top_10_em_df['symbol'] = symbol
        stock_gdfx_free_top_10_em_df['_id'] = symbol + '_' + stock_gdfx_free_top_10_em_df.index + '_' + period
        stock_gdfx_free_top_10_em_df['period'] = period
        mongodb_util.save_mongo(stock_gdfx_free_top_10_em_df, 'stock_gdfx_free_top_10')


def sync_stock_gdfx_free_top_10_one_day(str_day):
    real_time_quotes = east_money_stock_api.get_real_time_quotes_all_stocks()
    for real_time_one in real_time_quotes.itertuples():
        try:
            get_stock_gdfx_free_top_10_em(str_day, real_time_one.symbol)
        except BaseException as e:
            logger.error('同步所有股票前十大流通股本异常:{},{}', real_time_one.symbol, e)


from datetime import datetime

if __name__ == '__main__':
    now_date = datetime.now()
    str_day = now_date.strftime('%Y-%m-%d')
    logger.info('同步所有股票前十大流通股本')
    sync_stock_gdfx_free_top_10_one_day(str_day)



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 14
project_path = file_path[0:end]
sys.path.append(project_path)

from functools import lru_cache
from mns_common.db.MongodbUtil import MongodbUtil
import akshare as ak
from loguru import logger
import mns_common.utils.data_frame_util as data_frame_util

mongodb_util = MongodbUtil('27017')


def sync_trade_date():
    trade_date_list = ak.tool_trade_date_hist_sina()
    trade_date_list.trade_date = trade_date_list.trade_date.astype(str)
    trade_date_list['_id'] = trade_date_list['trade_date']
    trade_date_list['tag'] = False
    exist_trade_date_list = mongodb_util.find_all_data('trade_date_list')
    if data_frame_util.is_empty(exist_trade_date_list):
        mongodb_util.insert_mongo(trade_date_list, 'trade_date_list')
    else:
        new_trade_date_list = trade_date_list.loc[~(trade_date_list['_id'].isin(exist_trade_date_list['_id']))]
        if data_frame_util.is_not_empty(new_trade_date_list):
            mongodb_util.save_mongo(new_trade_date_list, 'trade_date_list')
        logger.info('同步交易日期任务完成')


# 获取上一个交易日期
@lru_cache(maxsize=None)
def get_last_trade_day(str_day):
    query = {'trade_date': {'$lt': str_day}}
    last_stock_zt_pool_group = mongodb_util.descend_query(query, 'trade_date_list', 'trade_date', 1)
    last_stock_zt_pool_group = last_stock_zt_pool_group.sort_values(by=['trade_date'],
                                                                    ascending=True)
    return list((last_stock_zt_pool_group.iloc[0:1])['trade_date'])[0]


# 前number个交易日 number=1 为当前时间
def get_before_trade_date(begin_day, number):
    query = {"_id": {'$lte': begin_day}}
    trade_date_list = mongodb_util.descend_query(query, 'trade_date_list', "_id", number)
    trade_date_list = trade_date_list.sort_values(by=['_id'],
                                                  ascending=True)
    before_days = list(trade_date_list[0:1]["_id"])[0]
    return before_days


# 获取未来第number个交易日
@lru_cache(maxsize=None)
def get_further_trade_date(begin_day, number):
    query = {"_id": {'$gte': begin_day}}
    trade_date_list = mongodb_util.ascend_query(query, 'trade_date_list', "_id", number)
    trade_date_list = trade_date_list.sort_values(by=['_id'],
                                                  ascending=False)
    before_days = list(trade_date_list[0:1]["_id"])[0]
    return before_days


def is_trade_day(str_day):
    query = {"trade_date": str_day}
    return mongodb_util.exist_data_query("trade_date_list", query)


if __name__ == '__main__':
    get_further_trade_date('2023-12-15', 5)



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 14
project_path = file_path[0:end]
sys.path.append(project_path)
import pandas as pd

def is_empty(df):
    if df is None or df.shape[0] == 0:
        return True
    else:
        return False


def is_not_empty(df):
    if df is None or df.shape[0] == 0:
        return False
    else:
        return True


def is_string_empty(s):
    if s is None:
        return True
    else:
        if len(s) == 0:
            return True
        else:
            return False


def is_string_not_empty(s):
    return bool(1 - is_string_empty(s))


def merge_choose_data_no_drop(df1, df2):
    if is_empty(df1) and is_empty(df2):
        return None
    if is_not_empty(df1) and is_empty(df2):
        return df1
    if is_empty(df1) and is_not_empty(df2):
        return df2

    return pd.concat([df1, df2])



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 14
project_path = file_path[0:end]
sys.path.append(project_path)

import socket


def get_host_ip():
    """
    查询本机ip地址
    :return:
    """
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 14
project_path = file_path[0:end]
sys.path.append(project_path)
import pandas as pd
import pymongo
from mns_common.utils.async_fun import async_fun
from loguru import logger
import mns_common.utils.ip_util as ip_util
import warnings

warnings.filterwarnings("ignore")


class MongodbUtil:
    def __init__(self, port):
        self.port = port

    def get_db(self):
        port = self.port
        if port == "remote":
            client = pymongo.MongoClient("mongodb://100.77.155.135:" + '27017' + "/patience")
        elif port == "remote1":
            client = pymongo.MongoClient("mongodb://100.77.155.135:" + '27019' + "/patience")
        # 家里
        elif '192.168.31' in ip_util.get_host_ip():
            client = pymongo.MongoClient("mongodb://127.0.0.1:" + port + "/patience")
        # 新都
        elif '192.168.68' in ip_util.get_host_ip():
            client = pymongo.MongoClient("mongodb://100.77.155.135:" + port + "/patience")
        else:
            client = pymongo.MongoClient("mongodb://100.77.155.135:" + port + "/patience")
        return client.patience

    def group(self, query, coll_name):
        db = self.get_db()
        collection = db[coll_name]
        data = collection.aggregate(query)
        return pd.DataFrame(list(data))

    def remove_data(self, query, coll_name):
        db = self.get_db()
        collection = db[coll_name]
        return collection.delete_many(query)

    def exist_data_query(self, coll_name, query):
        db = self.get_db()
        collection = db[coll_name]
        return collection.count(query, limit=1) > 0

    def find_one(self, coll_name, _id):
        db = self.get_db()
        collection = db[coll_name]
        return collection.find_one({'_id': _id})

    def find_one_query(self, coll_name, query):
        db = self.get_db()
        collection = db[coll_name]
        return pd.DataFrame(collection.find_one(query), index=[0])

    def find_all_data(self, coll_name):
        db = self.get_db()
        collection = db[coll_name]
        rows = collection.find({})
        df = pd.DataFrame([basic for basic in rows])
        return df

    def find_query_data(self, coll_name, query):
        db = self.get_db()
        collection = db[coll_name]
        rows = collection.find(query)
        df = pd.DataFrame(list(rows))
        return df

    def find_query_data_choose_field(self, coll_name, query, query_field):
        db = self.get_db()
        collection = db[coll_name]
        rows = collection.find(query, query_field)
        df = pd.DataFrame(list(rows))
        return df

    def find_query_data_list(self, coll_name, query):
        db = self.get_db()
        collection = db[coll_name]
        rows = collection.find(query)
        return list(rows)

    def find_query_data_list(self, coll_name, query):
        db = self.get_db()
        collection = db[coll_name]
        rows = collection.find(query)
        return list(rows)

    def remove_all_data(self, database):
        db = self.get_db()
        collection = db[database]
        query = {"_id": {"$ne": "null"}}
        collection.delete_many(query)

    def drop_collection(self, database):
        db = self.get_db()
        collection = db[database]
        collection.drop()

    def ascend_query(self, query, coll_name, field, num):
        db = self.get_db()
        collection = db[coll_name]
        return pd.DataFrame(list(collection.find(query).sort(field, 1).skip(0).limit(num)));

    def descend_query(self, query, coll_name, field, num):
        db = self.get_db()
        collection = db[coll_name]
        return pd.DataFrame(list(collection.find(query).sort(field, -1).skip(0).limit(num)));

    def count(self, query, coll_name):
        db = self.get_db()
        collection = db[coll_name]
        return collection.count_documents(query)

    def query_max(self, query, coll_name, field, num):
        db = self.get_db()
        collection = db[coll_name]
        return pd.DataFrame(list(collection.find(query).sort(field, -1).skip(0).limit(num)));

    def query_min(self, query, coll_name, field):
        db = self.get_db()
        collection = db[coll_name]
        return pd.DataFrame(list(collection.find(query).sort(field, 1).skip(0).limit(1)));

    def insert_mongo(self, df, database):
        db = self.get_db()
        if df is None or len(df) == 0:
            return
        collection = db[database]
        # 格式转换
        try:
            df = df.drop_duplicates()
            # df = df.T.drop_duplicates().T
            records = df.to_dict('records')
            collection.insert_many(records)
        except BaseException:
            logger.error("插入数据异常" + df)

    def insert_mongo_json(self, json, database):
        db = self.get_db()
        collection = db[database]
        # 格式转换
        try:
            collection.insert_many(json)
        except BaseException as e:
            logger.error("插入数据异常:{}", e)

    def save_mongo_json(self, json, database):
        db = self.get_db()
        collection = db[database]
        for record in json:
            try:
                collection.save(record)
            except BaseException as e:
                logger.error("保存数据出现异常:{}", e)

    def save_mongo(self, df, database):
        db = self.get_db()
        if df is None or len(df) == 0:
            return
        collection = db[database]
        # df = df.T.drop_duplicates().T
        # 格式转换
        records = df.to_dict('records')
        for record in records:
            try:
                collection.save(record)
            except BaseException as e:
                logger.error("保存数据出现异常:{},{}", record, e)

    def update_one(self, df, database):
        db = self.get_db()
        condition = {'_id': list(df['_id'])[0]}
        if len(df) == 0:
            return
        collection = db[database]
        collection.update(condition, df)

    def update_many(self, query, new_values, database):
        db = self.get_db()
        collection = db[database]
        x = collection.update_many(query, new_values)
        return x

    @async_fun
    def update_one_query(self, query, new_values, database):
        db = self.get_db()
        collection = db[database]
        x = collection.update(query, new_values)
        return x

    def distinct_field(self, database, field, query):
        db = self.get_db()
        collection = db[database]
        return collection.distinct(field, query)

    def create_index(self, database, index):
        db = self.get_db()
        collection = db[database]
        collection.create_index(
            index)

    def aggregate(self, pipeline, database):
        db = self.get_db()
        collection = db[database]
        data = collection.aggregate(pipeline)
        return pd.DataFrame(list(data))

    def get_col_keys(self, database):
        db = self.get_db()
        collection = db[database]
        keys = collection.find_one().keys()
        return keys


# if __name__ == '__main__':
#     symbol = '002992'
#     query = {'symbol': symbol,
#              '$and': [{'str_day': {'$gte': '2022-07-06'}}, {'str_day': {'$lte': '2022-11-06'}}]}
#     mongodb_util = MongodbUtil('27017')
#     # num = mongodb_util.count(query, 'stock_zt_pool')
#     # print(num)
#     key = mongodb_util.get_col_keys('stock_zt_pool')
#     print(key)
#
#     # num = mongodb_util.count(query, 'stock_zt_pool')
#     # print(num)
#
#     pipeline = [
#         {'$match': {
#             "classification": {'$in': ["K", "C"]},
#             "str_day": {'$gte': "2022-03-16"}}},
#         {'$group': {'_id': "$flow_mv_level", 'count': {'$sum': 1}}}
#     ]
#     result = mongodb_util.aggregate(pipeline, 'realtime_quotes_now_zt_new_kc_open')
#
#     result = result.sort_values(by=['_id'], ascending=True)
#     print(result)
from io import StringIO

if __name__ == '__main__':
    mongodb_util = MongodbUtil('27017')
    pipeline = [
        {'$match': {"yesterday_high_chg": False, "chg": {"$lte": 9}, "$and": [{"name": {"$not": {"$regex": ".*ST.*"}}},
                                                                              {"name": {"$not": {"$regex": ".*N.*"}}},
                                                                              {"name": {
                                                                                  "$not": {"$regex": ".*退.*"}}}]}},
        {'$group': {'_id': "$diff_days", 'count': {'$sum': 1}}}
    ]
    result = mongodb_util.aggregate(pipeline, 'realtime_quotes_now_zt_new_kc_open')

    result = result.sort_values(by=['_id'], ascending=True)
    print(result)

    # ths_new_concept = mongodb_util.find_all_data('ths_new_concept')
    key = mongodb_util.get_col_keys('company_info')
    print(key)

    # mongodb_util.create_index('realtime_quotes_now_open', [("number", 1)])
    # mongodb_util.create_index('realtime_quotes_now_open', [("symbol", 1), ("number", 1)])
    # mongodb_util.create_index('realtime_quotes_now_open', [("str_day", 1)])
    # update_query = {"str_day": "2023-06-30"}
    # mongodb_util.update_many(update_query, {"$set": {"number": 1}}, "realtime_quotes_now_open")
    query = {"symbol": "000617"}
    company_info_base = mongodb_util.find_query_data('company_info_base', query)
    ths_stock_concept_detail = mongodb_util.find_query_data('ths_stock_concept_detail', query)
    ths_stock_concept_detail = ths_stock_concept_detail[[
        'concept_code',
        'concept_name',
        'str_now_time',
        'concept_create_day']]
    # 去除空格
    ths_stock_concept_detail['concept_name'] = ths_stock_concept_detail['concept_name'].str.replace(' ', '')
    company_info_base.loc[:, 'ths_concept_list_info'] = ths_stock_concept_detail.to_string(index=False)
    for company_one in company_info_base.itertuples():
        ths_concept_list_info = company_one.ths_concept_list_info
        ths_concept_list_info_df = pd.read_csv(StringIO(ths_concept_list_info), delim_whitespace=True)
        print(ths_concept_list_info_df)



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 7
project_path = file_path[0:end]
sys.path.append(project_path)
import json

import requests
from pypushdeer import PushDeer


# token= AT_vL1h0mFSHtyRvbAED2mRozcEj8eFm4xh
def push_msg_to_wechat(title, msg):
    push = PushDeer(pushkey="PDU14860TJitHPGdTo3a7Gx0xMvV79aizrGXUs5No")
    push.send_text(title, desp=msg)

    url = 'http://wxpusher.zjiecode.com/api/send/message'

    s = json.dumps({'appToken': 'AT_vL1h0mFSHtyRvbAED2mRozcEj8eFm4xh',
                    'content': msg,
                    'summary': title,
                    'contentType': 1,
                    'uids': ['UID_vqzsUn5qXDROV2dbIZuZrioewJ9D']
                    })
    headers = {
        "Content-Type": "application/json"
    }

    requests.post(url, data=s, headers=headers)


if __name__ == '__main__':
    push_msg_to_wechat('test', 'big win')



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 7
project_path = file_path[0:end]
sys.path.append(project_path)
from enum import Enum


class Pattern_Enum(Enum):
    # 十字星
    CROSS_STAR = 'CROSS_STAR'
    # 下跌中的长上影线
    DOWN_OVER_LINING = 'DOWN_OVER_LINING'
    # 上涨中的长上影线
    UP_OVER_LINING = 'UP_OVER_LINING'
    # 高开低走
    OPEM_HIGH_AND_WALK_LOW = 'OPEM_HIGH_AND_WALK_LOW'

    OTHER = 'OTHER'



import sys
import os
import requests
import pandas as pd
import threading
from loguru import logger
import mns_common.api.kpl.symbol.kpl_symbol_common_field_constant as kpl_symbol_common_field_constant

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 14
project_path = file_path[0:end]
sys.path.append(project_path)
# 定义一个全局锁，用于保护 result 变量的访问
result_lock = threading.Lock()
# 初始化 result 变量为一个空的 Pandas DataFrame
result = pd.DataFrame()

MAX_PAGE_NUMBER = 60


# 定义一个函数用于获取接口数据
def fetch_data(page_number, end_date):
    global result
    index = page_number * MAX_PAGE_NUMBER

    # url = "https://apphq.longhuvip.com/w1/api/index.php?" \
    #       "Filter=0&FilterGem=0&FilterMotherboard=0&FilterTIB=0&Isst=0&Order=1&" \
    #       "PhoneOSNew=2&Ratio=6&Type=1&VerSion=5.11.0.3&a=RealRankingInfo_W8&apiv=w33&c=NewStockRanking&index=" \
    #       + str(index) + \
    #       "&st=" + str(MAX_PAGE_NUMBER)

    # url = "https://apphq.longhuvip.com/w1/api/index.php?Filter=0&FilterGem=0&FilterMotherboard=0" \
    #       "&FilterTIB=0&Isst=0&Order=1&PhoneOSNew=2&Ratio=6&Type=1&VerSion=5.11.0.3" \
    #       "&a=RealRankingInfo_W8&apiv=w33&c=NewStockRanking" \
    #       "&index=" + str(index) + \
    #       "&st=" + str(MAX_PAGE_NUMBER)

    url = "https://apphq.longhuvip.com/w1/api/index.php?Filter=0&FilterGem=0&FilterMotherboard=0" \
          "&FilterTIB=0&Isst=0&Order=1&PhoneOSNew=2&REnd=" + end_date + \
          "&RStart=0925&Ratio=6&Type=1&VerSion=5.11.0.3" \
          "&a=RealRankingInfo_W8&apiv=w33&c=NewStockRanking&" \
          "index=0" + str(index) + \
          "&st=" + str(MAX_PAGE_NUMBER)

    headers = {
        "Content-Type": "application/json; charset=utf-8"
    }

    r = requests.get(url, headers=headers)
    data_json = r.json()
    data_list = data_json['list']
    data_df = pd.DataFrame(data_list)

    with result_lock:
        # 使用锁来保护 result 变量的访问，将每页的数据添加到结果中
        result = pd.concat([result, data_df], ignore_index=True)


def get_stock_count(end_date):
    url = "https://apphq.longhuvip.com/w1/api/index.php?Filter=0&FilterGem=0&FilterMotherboard=0" \
          "&FilterTIB=0&Isst=0&Order=1&PhoneOSNew=2&REnd=" + end_date + \
          "&RStart=0925&Ratio=6&Type=1&VerSion=5.11.0.3" \
          "&a=RealRankingInfo_W8&apiv=w33&c=NewStockRanking&index=0&st=20"

    headers = {
        "Content-Type": "application/json; charset=utf-8"
    }

    r = requests.get(url, headers=headers)
    data_json = r.json()

    return data_json['Count']


# 获取当期交易股票的数量
def sync_real_time_quotes(count, end_date):
    global result
    result = pd.DataFrame()  # 重新初始化 result 变量
    threads = []
    page_number = round(count / MAX_PAGE_NUMBER, 0) + 1
    page_number = int(page_number)
    # 创建多个线程来获取数据
    for page in range(page_number):  # 0到100页
        thread = threading.Thread(target=fetch_data, args=(page, end_date,))
        threads.append(thread)
        thread.start()

    # 等待所有线程完成
    for thread in threads:
        thread.join()

    # 返回获取的接口数据

    result.columns = kpl_symbol_common_field_constant.SYNC_FIELD
    result = result[kpl_symbol_common_field_constant.CHOOSE_FIELD]

    return result


def get_kpl_real_time_quotes():
    try:
        end_date = '1500'
        symbol_count = get_stock_count(end_date)
        return sync_real_time_quotes(symbol_count, end_date)

    except BaseException as e:
        logger.error("同步开盘啦数据异常:{}", e)
        return None


#### todo 拿不到实时数据 只能五分钟拿一次
from mns_common.db.MongodbUtil import MongodbUtil

mongodb_util = MongodbUtil('27017')

if __name__ == '__main__':
    end_date = '1110'
    count = get_stock_count(end_date)
    number = 1
    while True:
        logger.info(number)
        result = sync_real_time_quotes(count, end_date)
        result = result.sort_values(by=['main_flow_net'], ascending=False)
        # mongodb_util.drop_collection('kpl_real_time_quotes')
        # mongodb_util.insert_mongo(result, 'kpl_real_time_quotes')
        logger.info(result)

        number = number + 1
